package fachada;

import util.Data;
import util.DataEmFormatoInvalidoException;
import util.DataInvalidaException;
import util.Periodo;
import util.excel.exception.CellNumberFormatException;
import util.excel.exception.CellStringFormatException;
import util.excel.exception.IOExcelException;
import util.excel.exception.NotDefinedCellException;
import util.excel.exception.NotDefinedRowException;
import util.excel.exception.SheetNotFoundException;
import negocios.CadastroQuartos;
import dados.IteratorQuartos;
import dados.IteratorReservas;
import dados.IteratorReservasArray;
import dados.Quarto;
import dados.RepositorioQuartos;
import dados.RepositorioQuartosArray;
import dados.RepositorioReservas;
import dados.RepositorioReservasArray;
import dados.Reserva;
import excecoes.QuartoJaCadastradoException;
import excecoes.QuartoLotadoException;
import excecoes.QuartoNaoCadastradoException;
import excecoes.ReservaJaCadastradaException;
import excecoes.ReservaNaoEncontradaException;
import excecoes.ValorInvalidoException;

public class Fachada {
	private CadastroQuartos servicosQuartos;
	private static Fachada fachada = null;
	
	public Fachada(RepositorioQuartos repositorio){
		this.servicosQuartos = new CadastroQuartos(repositorio);
	}
	
	public static Fachada getInstance(){
		if(fachada==null){
			fachada=new Fachada(new RepositorioQuartosArray());
		}
		return fachada;
	}
	
	public void inserirQuarto(Quarto quarto) throws QuartoJaCadastradoException{
		servicosQuartos.inserir(quarto);
	}
	
	public void removerQuarto(String numero) throws QuartoNaoCadastradoException {
		servicosQuartos.remover(numero);
	}
	
	public void inserirReserva(String numeroQuarto, Reserva reserva) throws QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException{
		servicosQuartos.inserirReserva(numeroQuarto, reserva);
	}	
	
	public Quarto procurarQuarto(String numero) throws QuartoNaoCadastradoException{
		return this.servicosQuartos.pesquisar(numero);
	}
	
	public void removerReserva(String numero, Reserva reserva) throws QuartoNaoCadastradoException, ReservaNaoEncontradaException {
		servicosQuartos.removerReserva(numero, reserva);
	}
	
	public void inserirEspecialmente(String endereco) throws IOExcelException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellNumberFormatException, CellStringFormatException, DataEmFormatoInvalidoException, QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException{
		servicosQuartos.insercaoEspecial(endereco);
	}
	
	public Quarto[] procurarQuartosDisponiveis(Periodo periodo, int quantidade){
		int proximo=0;
		Quarto[] arrayDisponivel = new Quarto[10];
		IteratorQuartos iterator = this.getIteratorQuartos();
		Quarto quarto = null;
		while(iterator.hasNext()){
			quarto = iterator.next();
			if(quarto.getVagas()>=quantidade) {
				if(!quarto.existeReserva(periodo)) {
					arrayDisponivel[proximo]=quarto;
					proximo++;
				}
			}
		}
		return arrayDisponivel;
		
	}
	
	public IteratorQuartos getIteratorQuartos(){
		return servicosQuartos.getIterator();
	}
	
	
	public double getFaturamentoPeriodo(Periodo periodo){
		double faturamentoPeriodo=0;
		IteratorQuartos iteratorQuartos = this.getIteratorQuartos();
		Quarto quarto = null;
		while(iteratorQuartos.hasNext()){
			quarto = iteratorQuartos.next();
			IteratorReservas iteratorReservas = quarto.getIteratorReservas();
			Reserva reserva = null;
			while(iteratorReservas.hasNext()){
				reserva = iteratorReservas.next();
				//if (reserva.getPeriodo().qtdDiasIntersecao(periodo)!=0)
				faturamentoPeriodo = faturamentoPeriodo + (reserva.getPeriodo().qtdDiasIntersecao(periodo)*quarto.getValor());
			}
		}
		
		return faturamentoPeriodo;
		
	}
	
	public static void main(String[] args) throws DataInvalidaException, ValorInvalidoException, QuartoJaCadastradoException, QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException, IOExcelException, ReservaNaoEncontradaException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellNumberFormatException, CellStringFormatException, DataEmFormatoInvalidoException {
		Fachada fachada = new Fachada(new RepositorioQuartosArray());
		
		/*	Data d1 = new Data(1,1,2007);
		 Data d2 = new Data(3,1,2007);
		 Data d3 = new Data(5,2,2007);
		 Data d4 = new Data(7,2,2007);
		 Data d5 = new Data(9,3,2007);
		 
		 
		 Periodo p1 = new Periodo(d1,d2);
		 Periodo p2 = new Periodo(d2,d3);
		 Periodo p3 = new Periodo(d1,d3);
		 Periodo p4 = new Periodo(d1,d4);
		 Periodo p5 = new Periodo(d2,d4);	
		 
		 
		 Quarto q1 = new Quarto("123",12,23);
		 Quarto q5 = new Quarto("123",12,23);
		 Quarto q2 = new Quarto("321",32,21);
		 Quarto q3 = new Quarto("111",11,11);
		 Quarto q4 = new Quarto("222",22,22);
		 
		 
		 
		 Reserva r1 = new Reserva(p3,12);
		 Reserva r2 = new Reserva(p2,12);
		 
		 fachada.inserirQuarto(q1);
		 fachada.inserirReserva("123",r1);
		 boolean b = fachada.procurarQuarto("123").existeReserva(r1.getPeriodo());
		 System.out.println(b);
		 
		 fachada.removerReserva("123",r1);
		 boolean c = fachada.procurarQuarto("123").existeReserva(r1.getPeriodo());
		 System.out.println(c);
		 
		 Data d6 = new Data(15,8,2006);
		 Data d7 = new Data(18,8,2007);	
		 Periodo p8 = new Periodo(d6,d7);
		 
		 Quarto centoEum = new Quarto("101",101,101);
		 
		 fachada.inserirQuarto(centoEum);
		 System.out.println(fachada.procurarQuarto("101").getVagas());
		 
		 fachada.inserirEspecialmente("./negocios/Reservas.xls");
		 boolean d= fachada.procurarQuarto("101").existeReserva(p8);
		 System.out.println(d);
		 */	
		
		Data d1 = new Data(1,1,2007);
		Data d2 = new Data(3,1,2007);
		Data d3 = new Data(4,1,2007);
		Data d4 = new Data(10,1,2007);
		
		Periodo p1 = new Periodo(d1,d2); //2dias
		Periodo p2= new Periodo(d2,d4); //7dias
		
		Periodo especial = new Periodo(d1,d4); //5dias
		
		Reserva r1 = new Reserva(p1,8);
	//	Reserva r2 = new Reserva(p2,8);
		
		Quarto quarto = new Quarto("123",10,100);
		fachada.inserirQuarto(quarto);
		
		fachada.inserirReserva("123",r1);
		//fachada.inserirReserva("123",r2);
		
		
		System.out.println(fachada.getFaturamentoPeriodo(especial));
		
		
		
		
	}
	
}