package negocios;

import util.Data;
import util.DataEmFormatoInvalidoException;
import util.Periodo;
import util.excel.ExcelCell;
import util.excel.ExcelRow;
import util.excel.ExcelSheet;
import util.excel.ExcelWorkbook;
import util.excel.exception.CellNumberFormatException;
import util.excel.exception.CellStringFormatException;
import util.excel.exception.IOExcelException;
import util.excel.exception.NotDefinedCellException;
import util.excel.exception.NotDefinedRowException;
import util.excel.exception.SheetNotFoundException;
import dados.IteratorQuartos;
import dados.Quarto;
import dados.RepositorioQuartos;
import dados.RepositorioQuartosArray;
import dados.Reserva;
import excecoes.QuartoJaCadastradoException;
import excecoes.QuartoLotadoException;
import excecoes.QuartoNaoCadastradoException;
import excecoes.ReservaJaCadastradaException;
import excecoes.ReservaNaoEncontradaException;

public class CadastroQuartos {

	private RepositorioQuartos quartos;

	public CadastroQuartos(RepositorioQuartos quartos){

		this.quartos = quartos;
	}

	public void insercaoEspecial(String endereco) throws IOExcelException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellNumberFormatException, CellStringFormatException, DataEmFormatoInvalidoException, QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException{
		ExcelWorkbook workbook = new ExcelWorkbook(endereco);
		ExcelSheet sheet = workbook.getSheet("Reservas");
		ExcelRow linha;
		int i =1;
		int hospedes;
		String numero;
		Periodo periodo;
		Data di;
		Data df;
		Reserva r;

		do{
			linha = sheet.getRow(i);
			numero = String.valueOf((int)linha.getCell(0).getNumericCellValue());
			hospedes = (int)linha.getCell(1).getNumericCellValue();			
			di = new Data(linha.getCell(2).getStringCellValue());
			df = new Data(linha.getCell(3).getStringCellValue());
			periodo = new Periodo(di,df);
			r = new Reserva(periodo,hospedes);
			this.inserirReserva(numero, r);
			i++;
		}while(!numero.equals(""));

	}

	public void inserir(Quarto quarto) throws QuartoJaCadastradoException {
		if(!existe(quarto))
			quartos.inserir(quarto);
		else
			throw new QuartoJaCadastradoException();
	}


	public Quarto pesquisar(String numero) throws QuartoNaoCadastradoException {
		Quarto resposta = quartos.pesquisar(numero);
		if (resposta == null)
			throw new QuartoNaoCadastradoException();
		else
			return resposta;
	}

	public void remover(String numero) throws QuartoNaoCadastradoException {
		Quarto q = this.pesquisar(numero);
		if(existe(q))
			quartos.remover(numero);
		else
			throw new QuartoNaoCadastradoException();
	}

	public void atualizar(Quarto quarto) throws QuartoNaoCadastradoException {
		if(existe(quarto))
			quartos.atualizar(quarto);
		else
			throw new QuartoNaoCadastradoException();
	}

	public boolean existe(Quarto quarto) {
		return quartos.existe(quarto);
	}

	public IteratorQuartos getIterator(){
		return quartos.getIterator();
	}

	public void inserirReserva(String numero, Reserva reserva) throws QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException{
		Quarto quarto = this.pesquisar(numero);
		if(quarto.getVagas() >= reserva.getNumeroHospedes()){
			quarto.inserirReserva(reserva);
			quarto.faturar(reserva.getPeriodo().qtdDias());
		} else
			throw new QuartoLotadoException(); 
	}

	public void removerReserva(String numero, Reserva reserva) throws QuartoNaoCadastradoException, ReservaNaoEncontradaException{
		Quarto quarto = this.pesquisar(numero);
		quarto.apagarReserva(reserva);
	}

	/*public static void main(String[] args) throws IOExcelException, SheetNotFoundException, NotDefinedRowException, NotDefinedCellException, CellNumberFormatException, CellStringFormatException, DataEmFormatoInvalidoException, QuartoNaoCadastradoException, ReservaJaCadastradaException, QuartoLotadoException {
		CadastroQuartos cad = new CadastroQuartos(new RepositorioQuartosArray());
		cad.insercaoEspecial("./negocios/Reservas.xls");
	}*/

}
