package util;

/**
 * Esta classe representa um objeto Data, com os elementos
 * Dia, Ms e Ano.
 * @author Nicole Sultanum e Victor Rodrigues
 *
 */
public class Data {
	private int dia;
	private int mes;
	private int ano;


	/**
	 * Construtor que recebe um dia, um ms e um ano para
	 * formar uma data. Por simplificao, todos os meses
	 * tm 30 dias, e datas fora de [1,9999] no so aceitas.
	 * Caso os parmetros sejam invlidos, uma exceo 
	 * lanada. 
	 * @param dia
	 * @param mes
	 * @param ano
	 */
	public Data(int dia, int mes, int ano) throws DataInvalidaException{
		//verificando dia
		if(dia > 0 && dia < 31){
			this.dia = dia;
		}else {
			throw new DataInvalidaException(dia, mes, ano);
		}
		//verificando ms
		if(mes > 0 && mes < 13){
			this.mes = mes;
		}else {
			throw new DataInvalidaException(dia, mes, ano);
		}
		//verificando ano
		if(ano > 0 && ano < 10000){
			this.ano = ano;
		}else {
			throw new DataInvalidaException(dia, mes, ano);
		}
	}

	/**
	 * Construtor que recebe uma string no formato dd/mm/aaaa, e cria uma data a partir dela.
	 * caso no esteja neste formato, uma exceo ser lanada.  
	 * @param data em String
	 */
	public Data(String s) throws DataEmFormatoInvalidoException{
		try{
			dia = Integer.parseInt(s.substring(0,2));
			mes = Integer.parseInt(s.substring(3,5));
			ano = Integer.parseInt(s.substring(6,10));
		}catch(NumberFormatException e){
			throw new DataEmFormatoInvalidoException(s);
		}catch(StringIndexOutOfBoundsException e){
			throw new DataEmFormatoInvalidoException(s);
		}
	}
	/**
	 * Retorna o ano armazenado nesta data
	 * @return
	 */
	public int getAno() {
		return ano;
	}

	/**
	 * Retorna o dia armazenado nesta data
	 * @return
	 */
	public int getDia() {
		return dia;
	}

	/**
	 * Retorna o ms armazenado nesta data
	 * @return
	 */
	public int getMes() {
		return mes;		
	}	

	/**
	 * Retorna 'true' se a data 'this' for anterior
	 *  data passada como parmetro.
	 * @param data a ser comparada
	 * @return comparao das datas
	 */
	public boolean anterior(Data t){
		boolean retorno = false;
		if(this.ano < t.ano ||
				(this.ano == t.ano && this.mes < t.mes) ||
				(this.ano == t.ano && this.mes == t.mes && this.dia < t.dia)){
			retorno = true;
		}
		return retorno;
	}
	/**
	 * Retorna 'true' se a data 'this' for posterior
	 *  data passada como parmetro.
	 * @param data a ser comparada
	 * @return comparao das datas
	 */
	public boolean posterior(Data t){
		return (!this.anterior(t) && !this.equals(t));
	}

	/**
	 * Verifica se a data 'this' e a data passada no parmetro
	 * so iguais.
	 * @param data a ser verificada
	 * @return true se as datas forem iguais.
	 */
	public boolean equals(Data t){
		return (this.dia == t.dia && this.mes == t.mes && this.ano == t.ano);
	}
	/**
	 * Retorna uma string representando a data.
	 * <b>Ateno!</b> No necessariamente este mtodo ir
	 * retornar a data no formato "dd/mm/aaaa".
	 */
	public String toString(){
		return dia + "/" + mes + "/" + ano;
	}

	public Data proximaData(){
		Data resposta = this;
		if(this.dia<30)		
			resposta.dia=this.dia+1;
		else if(this.mes<12) {
			resposta.dia = 1;
			resposta.mes=this.mes+1;
		}
		else {
			resposta.dia=1;
			resposta.mes=1;
			resposta.ano=this.ano+1;
		}
		
		return resposta;
	}

}
