package util;

/**
 * Classe que representa um perodo
 * 
 * @author Victor Rodrigues - vcr2@cin.ufpe.br
 *
 */
public class Periodo {
	
	private Data dataInicial;
	private Data dataFinal;
	
	/**
	 * Construtor de Perodo
	 * 
	 * Caso df < di, os parmetros so trocados
	 * 
	 * @param di Data Inicial
	 * @param df Data Final
	 */
	public Periodo(Data di, Data df){
		if(di.anterior(df)){
			this.dataInicial = di;
			this.dataFinal = df;
		} else {
			this.dataInicial = df;
			this.dataFinal = di;
		}
	}
	
	public String toString(){
		return this.getDataInicial().toString() +  " - " + this.getDataFinal().toString();
	}
	
	/**
	 * Mtodo para obteno da Data Inicial
	 * @return dataInicial
	 */
	public Data getDataInicial(){
		return this.dataInicial;
	}
	
	/**
	 * Mtodo que modifica a Data Inicial
	 * @param di Nova data inicial
	 * @throws PeriodoInvalidoException Caso dataFinal < di
	 */
	public void setDataInicial(Data di) throws PeriodoInvalidoException{
		if(di.anterior(this.dataFinal)){
			this.dataInicial = di;
		} else {
			throw new PeriodoInvalidoException();
		}
	}
	
	/**
	 * Mtodo para obteno da Data Final
	 * @return dataFinal
	 */
	public Data getDataFinal(){
		return this.dataFinal;
	}
	
	/**
	 * Mtodo que modifica a Data Final
	 * @param df Nova data final
	 * @throws PeriodoInvalidoException Caso dataInicial > df
	 */
	public void setDataFinal(Data df) throws PeriodoInvalidoException{
		if(df.posterior(this.dataInicial)){
			this.dataFinal = df;
		} else {
			throw new PeriodoInvalidoException();
		}
	}
	
	/**
	 * Mtodo que verifica se this e p so iguais
	 * @param p
	 * @return true se this e p so iguais; caso contrrio, false
	 */
	public boolean equals(Periodo p){
		boolean equals = false;
		if(this.getDataInicial().equals(p.getDataInicial())
				&& this.getDataFinal().equals(p.getDataFinal())){
			equals = true;
		}
		return equals;
	}
	
	/**
	 * Mtodo que verifica se this  anterior a p
	 * @param p
	 * @return true se this  anterior a p; caso contrrio, false
	 */
	public boolean anterior(Periodo p){
		boolean anterior = false;	
		if(this.getDataFinal().anterior(p.getDataInicial())){
			anterior = true;
		}
		return anterior;
	}
	
	/**
	 * Mtodo que verifica se this  posterior a p
	 * @param p
	 * @return true se this  posterior a p; caso contrrio, false
	 */
	public boolean posterior(Periodo p){
		boolean posterior = false;
		if(this.getDataInicial().posterior(p.getDataFinal())){
			posterior = true;
		}
		return posterior;
	}
	
	/**
	 * Mtodo que verifica se this possui interseo com p
	 * @param p
	 * @return true se this possui interseo com p; caso contrrio, false
	 */
	public boolean temIntersecao(Periodo p){
		boolean temIntersecao = false;
		if(!this.anterior(p) && !this.posterior(p)){
			temIntersecao = true;
		}		
		return temIntersecao;
	}
	
	/**
	 * Mtodo que verifica se this  subperodo de p
	 * @param p
	 * @return true se this  subperiodo de p; caso contrrio, false
	 */
	public boolean eSubPeriodo(Periodo p){
		boolean eSubPeriodo = false;
		if(this.getDataInicial().posterior(p.getDataInicial())
				&& this.getDataFinal().anterior(p.getDataFinal())){
			eSubPeriodo = true;
		}		
		return eSubPeriodo;
	}
	
	/**
	 * Mtodo que verifica se d est no perodo
	 * @param d
	 * @return true se d pertence ao perodo; caso contrrio, false
	 */
	public boolean estaNoPeriodo(Data d){
		boolean estaNoPeriodo = false;
		if(!this.getDataInicial().posterior(d) && !this.getDataFinal().anterior(d)){
			estaNoPeriodo = true;
		}
		return estaNoPeriodo;
	}
	
	
	public int qtdDias(){
		int i =0;
		
		while(!dataInicial.equals(dataFinal)){
			i++;
			dataInicial=dataInicial.proximaData();
			
		}
		
		return i;
	}
	
		public int qtdDiasIntersecao(Periodo p){
			
			int resposta;
			Data di;
			Data df;
			
			if(this.temIntersecao(p)){
				if(this.equals(p)) 
					resposta = p.qtdDias();
				else {
					if(p.dataInicial.anterior(this.dataInicial) || p.dataInicial.equals(this.dataInicial))
						di=this.dataInicial; 
					else di=p.dataInicial;
					
					if(p.dataFinal.posterior(this.dataFinal) || p.dataFinal.equals(this.dataFinal))
						df=this.dataFinal; 
					else df=p.dataFinal;
					
					Periodo teste=new Periodo(di,df);
					resposta = teste.qtdDias();
					
				}
				
			}
			else
				resposta=0;
			
			return resposta;
		}
			
}
